package android.lokemon.popups;

import java.util.ArrayList;
import java.util.Iterator;

import android.content.Context;
import android.content.Intent;
import android.lokemon.G;
import android.lokemon.R;
import android.lokemon.Util;
import android.lokemon.G.Mode;
import android.lokemon.game_objects.Pokemon;
import android.os.Bundle;
import android.util.Log;
import android.view.*;
import android.widget.*;
import com.commonsware.cwac.tlv.*;

public class PokemonPopup extends DragPopup implements TouchListView.DropListener{
	
	private ArrayList<Pokemon> entries;
	private EntryAdapter adapter;
	
	public void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		if (G.mode == Mode.MAP)
		{
			entries = G.player.pokemon;
			TouchListView tlv = (TouchListView)this.getListView();
			tlv.setDropListener(this);
		}
		else if (G.mode == Mode.BATTLE)
		{
			entries = new ArrayList<Pokemon>();
			for (Pokemon p:G.player.pokemon)
				if (p.getHP() > 0 && !p.inBattle) entries.add(p);
		}
		adapter = new EntryAdapter(this, R.layout.pokemon_item, entries);
		setListAdapter(adapter);
	}

	public void onListItemClick(ListView l, View v, int pos, long id)
	{
		if (G.mode == Mode.BATTLE || this.getIntent().getBooleanExtra("return_index", false))
		{
			Iterator<Pokemon> i = G.player.pokemon.iterator();
			int index = 0;
			Pokemon selected = entries.get(pos);
			while(i.hasNext())
			{
				if (i.next() == selected)
				{
					// returns the index of the pokemon in the player's inventory
					setResult(RESULT_FIRST_USER + index);
					break;
				}
				index++;
			}
			finish();
		}
		else if (G.mode == Mode.MAP)
		{
			// show moves popup
			Intent intent = new Intent(v.getContext(), MovesPopup.class);
			intent.putExtra("android.lokemon.PokeIndex", pos);
	        startActivity(intent);
		}
	}
	
	public void onPause()
	{
		super.onPause();
		this.setVisible(false);
	}
	
	public void onResume()
	{
		super.onResume();
		this.setVisible(true);
	}
	
	private class EntryAdapter extends ArrayAdapter<Pokemon>{
	
        private ArrayList<Pokemon> items;

        public EntryAdapter(Context context, int textViewResourceId, ArrayList<Pokemon> items) {
            super(context, textViewResourceId, items);
            this.items = items;
        }

        public View getView(int position, View convertView, ViewGroup parent) {
        	View v = convertView;
        	Pokemon entry = items.get(position);
        	if (v == null)
        	{
        		LayoutInflater vi = (LayoutInflater)getSystemService(Context.LAYOUT_INFLATER_SERVICE);
                v = vi.inflate(R.layout.pokemon_item, null);             
        	}
        	if (entry != null)
        	{
    			TextView name = (TextView)v.findViewById(R.id.name);
    			TextView hp = (TextView)v.findViewById(R.id.hp);
    			ProgressBar bar = (ProgressBar)v.findViewById(R.id.health_bar);
    			TextView type = (TextView)v.findViewById(R.id.type);
    			TextView attack = (TextView)v.findViewById(R.id.attack);
    			TextView defense = (TextView)v.findViewById(R.id.defense);
    			TextView speed = (TextView)v.findViewById(R.id.speed);
    			TextView special = (TextView)v.findViewById(R.id.special);
    			TextView level = (TextView)v.findViewById(R.id.level);
    			ImageView icon = (ImageView)v.findViewById(R.id.icon);
    			level.setText(entry.getLevel() + "");
    			name.setText(entry.getName());
    			hp.setText(entry.getHP() + "");
    			bar.setProgress((int)(entry.getHP()/(float)entry.getTotalHP()*100.0f));
    			String t = Util.capitalize((entry.getType1().name));
    			if (entry.getType2() != null)t += "/" + Util.capitalize((entry.getType2().name));
    			type.setText(t);
    			v.findViewById(R.id.col_1).setBackgroundColor(entry.getType1().colour_id);
    			if (entry.getType2() == null) 
    				v.findViewById(R.id.col_2).setBackgroundColor(entry.getType1().colour_id);
    			else
    				v.findViewById(R.id.col_2).setBackgroundColor(entry.getType2().colour_id);
    			attack.setText(entry.getAttack() + "");
    			defense.setText(entry.getDefense() + "");
    			special.setText(entry.getSpecial() + "");
    			speed.setText(entry.getSpeed() + "");
    			icon.setImageResource(entry.getSpriteNormal());
        	}
        	return v;
        }
	}

	// allow for dragging and dropping of pokemon
	public void drop(int from, int to) 
	{
		Pokemon entry = adapter.getItem(from);
		adapter.remove(entry);
		adapter.insert(entry, to);
		for (int i = 0; i < G.player.pokemon.size(); i++)
			G.player.pokemon.set(i, adapter.getItem(i));
	}
}
